@@ -165,7 +165,8 @@ meEVV <- function(data, z, prior = NULL, control = emControl(),
     # info <- c(iterations = its, error = err)
     dimnames(z) <- list(dimnames(data)[[1]], NULL)
     dimnames(mu) <- list(dimnames(data)[[2]], NULL)
-    dimnames(O) <- list(dimnames(data)[[2]], dimnames(data)[[2]], NULL)
+    dimnames(sigma) <- dimnames(O) <- 
+		    list(dimnames(data)[[2]], dimnames(data)[[2]], NULL)
     ## Sigma = scale * O %*% diag(shape) %*% t(O)
     variance <- list(modelName = "EVV", d = p, G = G, sigma = sigma,
                      scale = scale, shape = shape, orientation = O)
@@ -653,7 +654,8 @@ meVEE <- function(data, z, prior = NULL, control = emControl(),
     #                   inner = c(iterations = inner, error = inerr))
     dimnames(z) <- list(dimnames(data)[[1]],NULL)
     dimnames(mu) <- list(dimnames(data)[[2]], NULL)
-    dimnames(sigma) <- dimnames(O) <- list(dimnames(data)[[2]], dimnames(data)[[2]])
+    dimnames(O) <- list(dimnames(data)[[2]], dimnames(data)[[2]])
+    dimnames(sigma) <- list(dimnames(data)[[2]], dimnames(data)[[2]], NULL)
     ##  Sigma = scale * O %*% diag(shape) %*% t(O)
     variance <- list(modelName = "VEE", d = p, G = G, sigma = sigma,
                      scale = scale, shape = shape, orientation = O)
@@ -790,8 +792,8 @@ mstepVEE <- function(data, z, prior = NULL, warn = NULL, control = NULL, ...)
     info <- c(iteration = niterin, error = errin)
     dimnames(z) <- list(dimnames(data)[[1]], NULL)
     dimnames(mu) <- list(dimnames(data)[[2]], NULL)
-    dimnames(sigma) <- list(dimnames(data)[[2]], dimnames(data)[[2]])
     dimnames(O) <- list(dimnames(data)[[2]], dimnames(data)[[2]])
+    dimnames(sigma) <- list(dimnames(data)[[2]], dimnames(data)[[2]], NULL)
     variance <- list(modelName = "VEE", d = p, G = G, sigma = sigma,
                      scale = scale, shape = shape, orientation = O)
     parameters <- list(pro=pro, mean=mu, variance=variance)
@@ -1167,7 +1169,8 @@ meEVE <- function(data, z, prior = NULL, control = emControl(),
     #                   inner = c(iterations = inner, error = inerr))
     dimnames(z) <- list(dimnames(data)[[1]],NULL)
     dimnames(mu) <- list(dimnames(data)[[2]], NULL)
-    dimnames(sigma) <- dimnames(O) <- list(dimnames(data)[[2]], dimnames(data)[[2]])
+    dimnames(O) <- list(dimnames(data)[[2]], dimnames(data)[[2]])
+    dimnames(sigma) <- list(dimnames(data)[[2]], dimnames(data)[[2]], NULL)
     ##  Sigma = scale * O %*% diag(shape) %*% t(O)
     variance <- list(modelName = "EVE", d = p, G = G, sigma = sigma,
                      scale = scale, shape = shape, orientation = O)
@@ -1306,8 +1309,8 @@ mstepEVE <- function(data, z, prior = NULL, warn = NULL, control = NULL, ...)
     info <- c(iteration = niterin, error = errin)
     dimnames(z) <- list(dimnames(data)[[1]], NULL)
     dimnames(mu) <- list(dimnames(data)[[2]], NULL)
-    dimnames(sigma) <- list(dimnames(data)[[2]], dimnames(data)[[2]])
     dimnames(O) <- list(dimnames(data)[[2]], dimnames(data)[[2]])
+    dimnames(sigma) <- list(dimnames(data)[[2]], dimnames(data)[[2]], NULL)
     variance <- list(modelName = "EVE", d = p, G = G, sigma = sigma,
                      scale = scale, shape = shape, orientation = O)
     parameters <- list(pro=pro, mean=mu, variance=variance)
@@ -1684,7 +1687,8 @@ meVVE <- function(data, z, prior = NULL, control = emControl(),
     #                   inner = c(iterations = inner, error = inerr))
     dimnames(z) <- list(dimnames(data)[[1]],NULL)
     dimnames(mu) <- list(dimnames(data)[[2]], NULL)
-    dimnames(sigma) <- dimnames(O) <- list(dimnames(data)[[2]], dimnames(data)[[2]])
+    dimnames(O) <- list(dimnames(data)[[2]], dimnames(data)[[2]])
+    dimnames(sigma) <- list(dimnames(data)[[2]], dimnames(data)[[2]], NULL)
     ##  Sigma = scale * O %*% diag(shape) %*% t(O)
     variance <- list(modelName = "VVE", d = p, G = G, sigma = sigma,
                      scale = scale, shape = shape, orientation = O)
@@ -1823,8 +1827,8 @@ mstepVVE <- function(data, z, prior = NULL, warn = NULL, control = NULL, ...)
     info <- c(iteration = niterin, error = errin)
     dimnames(z) <- list(dimnames(data)[[1]], NULL)
     dimnames(mu) <- list(dimnames(data)[[2]], NULL)
-    dimnames(sigma) <- list(dimnames(data)[[2]], dimnames(data)[[2]])
     dimnames(O) <- list(dimnames(data)[[2]], dimnames(data)[[2]])
+    dimnames(sigma) <- list(dimnames(data)[[2]], dimnames(data)[[2]], NULL)
     variance <- list(modelName = "VVE", d = p, G = G, sigma = sigma,
                      scale = scale, shape = shape, orientation = O)
     parameters <- list(pro=pro, mean=mu, variance=variance)
